@@ -86,7 +86,7 @@ typedef struct {
 typedef struct {
     char *data;
     char *cur;
-    int dataSize;
+    uint32_t dataSize;
     bson_bool_t finished;
     int stack[32];
     int stackPos;
@@ -119,7 +119,7 @@ typedef struct {
  *
  * @return the size.
  */
-int bson_size( const bson *b );
+uint32_t bson_size( const bson *b );
 
 /**
  * Print a string representation of a BSON object.
@@ -546,7 +546,7 @@ int bson_init_finished_data( bson *b, char *data );
  *
  * @return BSON_OK or BSON_ERROR.
  */
-void bson_init_size( bson *b, int size );
+void bson_init_size( bson *b, uint32_t size );
 
 /**
  * Grow a bson object.
@@ -557,7 +557,7 @@ void bson_init_size( bson *b, int size );
  * @return BSON_OK or BSON_ERROR with the bson error object set.
  *   Exits if allocation fails.
  */
-int bson_ensure_space( bson *b, const int bytesNeeded );
+int bson_ensure_space( bson *b, const uint32_t bytesNeeded );
 
 /**
  * Finalize a bson object.
@@ -930,7 +930,7 @@ extern bson_printf_func bson_errprintf;
  *
  * @sa malloc(3)
  */
-void *bson_malloc( int size );
+void *bson_malloc( uint32_t size );
 
 /**
  * Changes the size of allocated memory and checks return value,
@@ -943,7 +943,7 @@ void *bson_malloc( int size );
  *
  * @sa realloc()
  */
-void *bson_realloc( void *ptr, int size );
+void *bson_realloc( void *ptr, uint32_t size );
 
 /**
  * Set a function for error handling.
