@@ -70,9 +70,17 @@ static const char* username = "whoopsie";
  *  - The MAC address of the first non-loopback device, SHA-512 hashed */
 static char* whoopsie_identifier = NULL;
 
+/*  Whether or not to verify the ssl peer when submitting crashes. Either:
+ *  - 0 verification will NOT occur
+ *  - 1 verification will occur */
+static long verifypeer = 1;
+
 /* The URL for sending the initial crash report */
 static char* crash_db_submit_url = NULL;
 
+/* OOPS ID of the last successfully-uploaded crash file */
+char* last_uploaded_oopsid = NULL;
+
 /* The file path and descriptor for our instance lock */
 static const char* lock_path = "/var/lock/whoopsie/lock";
 static int lock_fd = 0;
@@ -170,6 +178,8 @@ static const char* acceptable_fields[] = {
     "SysLog",
     /* openjdk-8 */
     "HotspotError",
+    /* desktop app debugging info */
+    "JournalErrors",
 
     NULL,
 };
@@ -230,8 +240,8 @@ append_key_value (gpointer key, gpointer value, gpointer bson_string)
 size_t
 server_response (char* ptr, size_t size, size_t nmemb, void* s)
 {
-    struct response_string* resp = (struct response_string*) s;
-    grow_response_string (resp, ptr, size * nmemb);
+    GString *resp = (GString *) s;
+    g_string_append (resp, ptr);
     return size * nmemb;
 }
 
@@ -251,7 +261,7 @@ split_string (char* head, char** tail)
 
 gboolean
 bsonify (GHashTable* report, bson* b, const char** bson_message,
-         int* bson_message_len)
+         uint32_t* bson_message_len)
 {
     /* Attempt to convert a #GHashTable of the report into a BSON string.
      * On error return %FALSE. */
@@ -285,7 +295,7 @@ bsonify (GHashTable* report, bson* b, const char** bson_message,
 }
 
 int
-upload_report (const char* message_data, int message_len, struct response_string* s)
+upload_report (const char* message_data, uint32_t message_len, GString *s)
 {
     CURL* curl = NULL;
     CURLcode result_code = 0;
@@ -316,6 +326,7 @@ upload_report (const char* message_data, int message_len, struct response_string
     curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, server_response);
     curl_easy_setopt (curl, CURLOPT_WRITEDATA, s);
     curl_easy_setopt (curl, CURLOPT_VERBOSE, 0L);
+    curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, verifypeer);
 
     result_code = curl_easy_perform (curl);
     curl_slist_free_all(list);
@@ -363,8 +374,8 @@ parse_report (const char* report_path, gboolean full_report, GError** error)
     gchar* value_p = NULL;
     GError* err = NULL;
     gchar* end = NULL;
-    int value_length;
-    int value_pos;
+    size_t value_length;
+    size_t value_pos;
 
     g_return_val_if_fail (report_path, NULL);
 
@@ -551,7 +562,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     long response_code = 0;
     struct curl_slist* list = NULL;
     char* crash_db_core_url = NULL;
-    struct response_string s;
+    GString *s = NULL;
 
 
     g_return_val_if_fail (uuid, FALSE);
@@ -569,7 +580,7 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
         g_free (crash_db_core_url);
         return FALSE;
     }
-    init_response_string (&s);
+    s = g_string_new (NULL);
     curl_easy_setopt (curl, CURLOPT_POST, 1);
     curl_easy_setopt (curl, CURLOPT_NOPROGRESS, 1);
     list = curl_slist_append (list, "Content-Type: application/octet-stream");
@@ -578,8 +589,9 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     curl_easy_setopt (curl, CURLOPT_HTTPHEADER, list);
     curl_easy_setopt (curl, CURLOPT_POSTFIELDS, (void*)core_data);
     curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, server_response);
-    curl_easy_setopt (curl, CURLOPT_WRITEDATA, &s);
+    curl_easy_setopt (curl, CURLOPT_WRITEDATA, s);
     curl_easy_setopt (curl, CURLOPT_VERBOSE, 0L);
+    curl_easy_setopt (curl, CURLOPT_SSL_VERIFYPEER, verifypeer);
 
     result_code = curl_easy_perform (curl);
     curl_slist_free_all(list);
@@ -591,11 +603,25 @@ upload_core (const char* uuid, const char* arch, const char* core_data) {
     log_msg ("Response code: %ld\n", response_code);
     curl_easy_cleanup (curl);
     g_free (crash_db_core_url);
-    destroy_response_string (&s);
+    g_string_free (s, TRUE);
 
     return result_code == CURLE_OK && response_code == 200;
 }
 
+void unset_last_uploaded_oopsid ()
+{
+    if (last_uploaded_oopsid) {
+        g_free (last_uploaded_oopsid);
+        last_uploaded_oopsid = NULL;
+    }
+}
+
+void set_last_uploaded_oopsid (char* response)
+{
+    unset_last_uploaded_oopsid();
+    last_uploaded_oopsid = g_strdup_printf ("%.36s", response);
+}
+
 void
 handle_response (GHashTable* report, char* response_data)
 {
@@ -612,6 +638,7 @@ handle_response (GHashTable* report, char* response_data)
     if (command) {
         if (strcmp (command, "CORE") == 0) {
             log_msg ("Reported OOPS ID %.36s\n", response_data);
+            set_last_uploaded_oopsid(response_data);
             core = g_hash_table_lookup (report, "CoreDump");
             arch = g_hash_table_lookup (report, "Architecture");
             if (core && arch) {
@@ -622,10 +649,12 @@ handle_response (GHashTable* report, char* response_data)
                     log_msg ("Upload of the core dump failed.\n");
             } else if (strcmp (command, "OOPSID") == 0) {
                 log_msg ("Reported OOPS ID %.36s\n", response_data);
+                set_last_uploaded_oopsid(response_data);
             } else
                 log_msg ("Asked for a core dump that we don't have.\n");
         } else if (strcmp (command, "OOPSID") == 0) {
             log_msg ("Reported OOPS ID %.36s\n", response_data);
+            set_last_uploaded_oopsid(response_data);
         } else
             log_msg ("Got command: %s\n", command);
     }
@@ -636,9 +665,8 @@ parse_and_upload_report (const char* crash_file)
 {
     GHashTable* report = NULL;
     gboolean success = FALSE;
-    int message_len = 0;
+    uint32_t message_len = 0;
     const char* message_data = NULL;
-    struct response_string s;
     GError* error = NULL;
     bson b[1];
     int response = 0;
@@ -665,8 +693,8 @@ parse_and_upload_report (const char* crash_file)
         success = TRUE;
     } else {
         log_msg ("Uploading %s.\n", crash_file);
-        init_response_string (&s);
-        response = upload_report (message_data, message_len, &s);
+        GString* s = g_string_new (NULL);
+        response = upload_report (message_data, message_len, s);
         if (bson_data (b))
             bson_destroy (b);
 
@@ -681,12 +709,13 @@ parse_and_upload_report (const char* crash_file)
 
         if (response > 200) {
             log_msg ("Server replied with:\n");
-            log_msg ("%s\n", s.p);
+            log_msg ("%s\n", s->str);
         }
 
-        if (response == 200 && s.length > 0)
-            handle_response (report, s.p);
-        destroy_response_string (&s);
+        if (response == 200 && s->len > 0)
+            handle_response (report, s->str);
+
+        g_string_free (s, TRUE);
     }
 
     g_hash_table_foreach (report, destroy_key_and_value, NULL);
@@ -728,7 +757,7 @@ process_existing_files (const char* report_dir)
             g_free (crash_file);
             continue;
         } else if (online_state && parse_and_upload_report (crash_file)) {
-            if (!mark_handled (crash_file)) {
+            if (!mark_handled (crash_file, last_uploaded_oopsid)) {
                 log_msg ("Unable to mark report as seen (%s) removing it.\n", crash_file);
                 g_unlink (crash_file);
             }
@@ -999,6 +1028,12 @@ main (int argc, char** argv)
     else
         whoopsie_identifier_generate (&whoopsie_identifier, &err);
 
+    env = g_getenv ("CRASH_DB_NOVERIFYPEER");
+    if (env != NULL) {
+        log_msg ("CRASH_DB_NOVERIFYPEER is set, not verifying ssl cert.\n");
+        verifypeer = 0;
+    }
+
     if (err) {
         log_msg ("%s\n", err->message);
         g_error_free (err);
